# How to Build Tinder

## Class 0 - React Native vs. Expo

  * UPDATES
    * React Native - has more up to date code with iOS firmware
    * Expo - less updates but cleaner no breaking code
  * FLEXIBILITY
    * React Native - emulates more native code and features
    * Expo - has less features but they work better
  * CODEBASE
    * React Native - Android & iOS have different codebases but 85% can usually be copy and pasted into iOS
    * Expo - One code base
        * exp build:ios & exp build:android (AWESOME)
  * NOTIFICATIONS
    * React Native - create your own server, much more complicated
    * Expo - provided framework out of the box to allow and use notifications
  * LOGIN
    * React Native - BYOL Bring Your Own Login. Can use any service but must implement and figure it out on your own
    * Expo - Less options but exposes minimal native API for Facebook & Google (can also easily implement firebase on both)
  * TESTING
    * React Native - Must go through apple to add test users with Apple account
    * Expo - Use ecosystem to test through expo app on both iOS & Android
  * DEV TEAM
    * React Native - Facebook’s huge dev team constantly implementing new features
    * Expo - A lot of the devs work open source but have previously built a lot of the existing React Native modules










---------------------------------------------------------------------------------










## Class 1 - Getting Started With Expo

[Download Expo iPhone App](https://xde-updates.exponentjs.com/download/mac)
[Download Xcode](https://itunes.apple.com/app/xcode/id497799835)
[Download Node](https://nodejs.org/en/)
[Download Watchman](https://facebook.github.io/watchman/docs/install.html)
[Expo Documentation](https://docs.expo.io/versions/latest/introduction/installation.html)

```
npm install -g exp
npm install -g create-react-native-app
create-react-native-app Tinder
cd Tinder
npm start or yarn start 
```

Developer Tools Simulator (command + D)











---------------------------------------------------------------------------------













## Class 2 - Organize Folder Structure

FOLDER STRUCTURE
  expo - (default)
  assets - (images, logos, icons, fonts, splashscreens)
  components (modules that arent pages used in multple areas)
  config - (credentials for AWS & Firebase)
  navigation - (react navigation struture)
  node_modules - (default)
  redux - (actions & reducers)
  screens - (Chat, Home, Login, Matches, Profile)
App.js - (Initial page called on app startup)
styles.js - (overall app styling)
app.json - (settings for xCode & expo)
package.json - (file for app info and node modules)

Test.js
```
import React from 'react';
import styles from '../styles'

import { 
  Text, 
  View
} from 'react-native';

class Chat extends React.Component {
  state = {}

  componentWillMount() {}

  render() {
    return (
     <View>
      <Text>Test</Text>
     </View>
    )
  }
}

export default Test;

```

styles.js
```
import React from 'react';
import { StyleSheet } from 'react-native';

var styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: '#fff',
    alignItems: 'center',
    justifyContent: 'center',
  },
})

module.exports = styles
``









---------------------------------------------------------------------------------











## Video 3 - Add Navigation

[React Navigation Documentation](https://reactnavigation.org/docs/intro/)
[Options for TabNavigator](https://reactnavigation.org/docs/navigators/tab/)

```
npm install react-navigation --save
```

TabNavigator.js
```
import React from 'react';
import Home from '../screens/Home';
import Profile from '../screens/Profile';
import Matches from '../screens/Matches';
import { TabNavigator } from 'react-navigation';

export default TabNavigator(
  {
    Profile: {
      screen: Profile,
      navigationOptions: {
        tabBarLabel: 'Profile',
      },
    },
    Home: {
      screen: Home,
      navigationOptions: {
        tabBarLabel: 'Home',
      }
    },
    Matches: {
      screen: Matches,
      navigationOptions: {
        tabBarLabel: 'Matches',
      },
    },
  },
  {
    navigationOptions: {
      header: null
    },
    tabBarPosition: 'top',
    initialRouteName: 'Home',
    animationEnabled: true,
    swipeEnabled: false,
    tabBarOptions: {
      style: {
        height: 100,
        backgroundColor: '#fff',
      },
    }
  }
);
```

RootNavigation.js
```
import React from 'react';
import { StackNavigator } from 'react-navigation';
import MainTabNavigator from './MainTabNavigator';

const RootStackNavigator = StackNavigator(
  {
    Main: {
      screen: MainTabNavigator,
    },
  }
);

export default class RootNavigator extends React.Component {
  render() {
    return <RootStackNavigator/>;
  }
}
```







-----------------------------------------------------------------------------------------------------------









## Video 4 - Redux Integration

```
npm install --save redux redux-thunk react-redux 
```
actions.js
```
export function login(){
  return function(dispatch){
    dispatch({ type: 'LOGIN', payload: 'test' });
  }
}
```

reducers.js
```
export default reducers = (state = {
    user: '',
  }, action) => {
    switch (action.type) {
      case 'LOGIN': {
        return { ...state, user: action.payload }
      }
    }
    return state;
} 
```

App.js
```
import React from 'react';
import Login from './screens/Login';
import reducers from './redux/reducers';
import thunkMiddleware from 'redux-thunk';
import { Provider } from 'react-redux';
import { createStore, applyMiddleware } from 'redux';
const middleware = applyMiddleware(thunkMiddleware)
const store = createStore(reducers, middleware);

export default class App extends React.Component {
  render() {
    return (
      <Provider store={store}>
        <Login/>
      </Provider>
    );
  }
}
```

Import connect into component
```
import { connect } from 'react-redux';
import { login } from '../redux/actions'
```

Use connect in components
```
function mapStateToProps(state) {
  return {
    user: state.user
  };
}
```

Export connect enabled component
```
export default connect(mapStateToProps)(Test);
```

Call redux function
```
this.props.dispatch(login())
```




-----------------------------------------------------------------------------------------------------------
































































## Video 2 - Create Facebook Login

[Create Facebook Developer Account](https://developers.facebook.com/async/onboarding/dialog/)
[Facebook Login Quickstart](https://developers.facebook.com/docs/facebook-login/ios)
[Facebook Apps Dashboard](https://developers.facebook.com/apps)
[Expo Facebook Login Documentation](https://docs.expo.io/versions/latest/sdk/facebook.html)


Registering App with Facebook
Bundle ID -  host.exp.Exponent
Android Key - rRW++LUjmZZ+58EbN5DVhGAnkX4=

App.json
```
{
  "expo": {
    "sdkVersion": "22.0.0",
    "facebookScheme": "fbAPPID",
    "FacebookAppID":"APPID",
    "facebookDisplayName": "Tinder"
  }
}
```

App.js - Login function 
```
login = async () => {
  const { type, token } = await Expo.Facebook.logInWithReadPermissionsAsync('APPID', {
      permissions: ['public_profile'],
    });
  if (type === 'success') {
    // Get the user's name using Facebook's Graph API
    const response = await fetch(
      `https://graph.facebook.com/me?access_token=${token}`);
    Alert.alert(
      'Logged in!',
      `Hi ${(await response.json()).name}!`,
    );
  }
}  
```










## Video 4 - Firebase/Profile Page

[Signup for Firebase](https://console.firebase.google.com)
[Expo Firebase Documentation](https://docs.expo.io/versions/latest/guides/using-firebase.html)

Install Firebase
```
npm install --save firebase
```

Setup Firebase
```
import * as firebase from 'firebase';

// Initialize Firebase
const firebaseConfig = {
  apiKey: "<YOUR-API-KEY>",
  authDomain: "<YOUR-AUTH-DOMAIN>",
  databaseURL: "<YOUR-DATABASE-URL>",
  storageBucket: "<YOUR-STORAGE-BUCKET>"
};

firebase.initializeApp(firebaseConfig);
```


Login With Facebook and store user with Firebase
```
  const { type, token } = await Expo.Facebook.logInWithReadPermissionsAsync(
    '1576176899112755',
    { permissions: ['public_profile'] }
  );

  if (type === 'success') {
    // Build Firebase credential with the Facebook access token.
    const credential = await firebase.auth.FacebookAuthProvider.credential(token);

    // Sign in with credential from the Facebook user.
    firebase.auth().signInWithCredential(credential).catch((error) => {
      // Handle Errors here.
      Alert.alert("Try Again")
    });
  }
```


Check if user has already authenticated
```
firebase.auth().onAuthStateChanged((user) => {
  if (user != null) {
    this.setState({ loggedIn: true });
    console.log("We are authenticated now!" + JSON.stringify(user));
  }
});
```




## Video 5 - Tinder Cards

[React Native Tinder Card Module](https://github.com/meteor-factory/react-native-tinder-swipe-cards)

Install Tinder Cards
```
npm install --save react-native-swipe-cards
Create a module e.g. SwipeCards.js
Import it import SwipeCards from './SwipeCards.js'
Render it <SwipeCards style={{flex: 1}} />
```

SwipeCards.js
```
class Card extends React.Component {
  render() {
    return (
      <View style={styles.card}>
        <Image style={styles.thumbnail} source={{uri: this.props.image}} />
        <Text>{this.props.name}</Text>
      </View>
    )
  }
}

class NoMoreCards extends React.Component {
  render() {
    return (
      <View>
        <Text>No more cards</Text>
      </View>
    )
  }
}

export default class extends React.Component {
  state = {
    cards: [
      {name: 'fran', image: 'https://media.giphy.com/media/GfXFVHUzjlbOg/giphy.gif'},
      {name: 'jackie', image: 'https://media.giphy.com/media/irTuv1L1T34TC/giphy.gif'},
      {name: 'phil', image: 'https://media.giphy.com/media/LkLL0HJerdXMI/giphy.gif'},
      {name: 'jacks', image: 'https://media.giphy.com/media/fFBmUMzFL5zRS/giphy.gif'},
      {name: 'mellow', image: 'https://media.giphy.com/media/oDLDbBgf0dkis/giphy.gif'},
      {name: 'frank', image: 'https://media.giphy.com/media/7r4g8V2UkBUcw/giphy.gif'},
      {name: 'timmmay', image: 'https://media.giphy.com/media/K6Q7ZCdLy8pCE/giphy.gif'},
    ]
  };

  handleYup (card) {
    console.log(`Yup for ${card.name}`)
  }
  handleNope (card) {
    console.log(`Nope for ${card.name}`)
  }
  handleMaybe (card) {
    console.log(`Maybe for ${card.name}`)
  }
  render() {
    return (
      <SwipeCards
        cards={this.state.cards}
        stack={false}
        renderCard={(cardData) => <Card {...cardData} />}
        renderNoMoreCards={() => <NoMoreCards />}
        showYup={false}
        showNope={false}
        handleYup={this.handleYup}
        handleNope={this.handleNope}
        handleMaybe={this.handleMaybe}
        hasMaybeAction={false}/>
    )
  }
}
```




## Video 6 - Profile Update

[Expo Image Picker](https://docs.expo.io/versions/latest/sdk/imagepicker.html)

Add the Imagepicker Module from Expo
```
import { ImagePicker } from 'expo';

pickImage = async () => {
  let result = await ImagePicker.launchImageLibraryAsync({
    allowsEditing: true,
    aspect: [4, 3]
  });
  console.log(result.uri) //this is the response of the local image file, we need to convert this to a link to post in firebase
}
```

Use React Native AWS3 library to host images

[AWS S3 Image Upload](https://github.com/benjreinhart/react-native-aws3)

```
import { RNS3 } from 'react-native-aws3';

const file = {
  uri: uri,
  name: "image.png",
  type: "image/png"
}

const options = {
  keyPrefix: "uploads/",
  bucket: "tindertutorial",
  region: "us-east-1",
  accessKey: aws.accessKey, //use your own s3 access key here
  secretKey: aws.secretKey, //use your own s3 secret key here
  successActionStatus: 201 
}

RNS3.put(file, options).then(response => {
  if (response.status !== 201)
    throw new Error("Failed to upload image to S3");
  console.log(response.body.postResponse.location) //this is the response url you get back from aws to post in firebase
});
```




## Video 8 - Chat Feature

[React Native Gifted Chat](https://github.com/FaridSafi/react-native-gifted-chat)

npm install react-native-gifted-chat --save

```
import { GiftedChat } from 'react-native-gifted-chat'

state = {
  messages: [],
}

componentWillMount() {
  this.setState({
    messages: [
      {
        _id: 1,
        text: 'Hello developer',
        createdAt: new Date(),
        user: {
          _id: 2,
          name: 'React Native',
          avatar: 'https://facebook.github.io/react/img/logo_og.png',
        },
      },
    ],
  })
}

onSend(messages = []) {
  this.setState(previousState => ({
    messages: GiftedChat.append(previousState.messages, messages),
  }))
}

render() {
  return (
    <GiftedChat
      messages={this.state.messages}
      onSend={messages => this.onSend(messages)}
      user={{
        _id: 1,
      }}
    />
  )
}
```




## Video 10 - Location Services

[NPM Geohash](https://www.npmjs.com/package/latlon-geohash)

npm install --save latlon-geohash

```
import { Constants, Location, Permissions } from 'expo';

  getLocationAsync = async () => {
    let { status } = await Permissions.askAsync(Permissions.LOCATION);
    if (status !== 'granted') {
      console.log("Permission was denied")
    }

    let location = await Location.getCurrentPositionAsync({});
    let geocode = Geohash.encode(location.coords.latitude, location.coords.longitude, '5')
    await AsyncStorage.setItem('geocode', geocode);
    this.update(geocode)
  };

  update(geocode){
    let params = {
      id: firebase.auth().currentUser.uid,
      photoUrl: firebase.auth().currentUser.photoURL,
      name: firebase.auth().currentUser.displayName,
      email: firebase.auth().currentUser.email,
      geocode: geocode
    }
    firebase.database().ref('cards/' + firebase.auth().currentUser.uid ).update(params);
  }
```

Require AsyncStorage from React Native
```
  try {
    await AsyncStorage.setItem('@MySuperStore:key', 'I like to save it.');
  } catch (error) {
    // Error saving data
  }
  Fetching data:

  try {
    const value = await AsyncStorage.getItem('@MySuperStore:key');
    if (value !== null){
      // We have data!!
      console.log(value);
    }
  } catch (error) {
    // Error retrieving data
  }
```

#   km      
1   ±2500
2   ±630
3   ±78
4   ±20
5   ±2.4
6   ±0.61
7   ±0.076
8   ±0.019
9   ±0.0024
10  ±0.00060
11  ±0.000074





## Video 11 - Push Notifications

[Expo Push Notifications](https://docs.expo.io/versions/latest/guides/push-notifications.html)

App.js
```
import { 
  Permissions,
  Notifications 
} from 'expo';

  componentWillMount() {
    this.registerForPushNotificationsAsync();
  }

  registerForPushNotificationsAsync = async () => {
    const { status: existingStatus } = await Permissions.getAsync(Permissions.NOTIFICATIONS);

    let finalStatus = existingStatus;

    // only ask if permissions have not already been determined, because
    // iOS won't necessarily prompt the user a second time.
    if (existingStatus !== 'granted') {
      // Android remote notification permissions are granted during the app
      // install, so this will only ask on iOS
      const { status } = await Permissions.askAsync(Permissions.NOTIFICATIONS);
      finalStatus = status;
    }

    // Stop here if the user did not grant permissions
    if (finalStatus !== 'granted') {
      return;
    }

    // Get the token that uniquely identifies this device
    let token = await Notifications.getExpoPushTokenAsync();
    firebase.database().ref('cards/' + firebase.auth().currentUser.uid ).update({token: token});
  }
```

SwipeCards.js
```
  handleYup (card) {
    this.sendNotification(card);
  }

  sendNotification(card){
    const PUSH_ENDPOINT = 'https://exp.host/--/api/v2/push/send';
    console.log("send notification" + card.token)
    return fetch(PUSH_ENDPOINT, {
      method: 'POST',
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
      },
      body: JSON.stringify({
        to: card.token,
        title: "It's A Match!",
        badge: 1,
        body: "Message " + card.name + " and say hi!"
      }),
    });
  }

  //IN THE RENDER METHOD
  <SwipeCards
    cards={this.state.cards}
    stack={false}
    renderCard={(cardData) => <Card {...cardData} />}
    renderNoMoreCards={() => <NoMoreCards />}
    showYup={false}
    showNope={false}
    showMaybe={false}
    handleYup={this.handleYup.bind(this)}
    handleNope={this.handleNope}
    handleMaybe={this.handleMaybe}
    hasMaybeAction={true}/>
```



## Video 12 - Publish to Expo

[App Icons Expo](https://docs.expo.io/versions/latest/guides/app-icons.html)
[Splash Screens Expo](https://docs.expo.io/versions/latest/guides/splash-screens.html)
[Publish to Expo](https://docs.expo.io/versions/latest/guides/publishing.html)

Splash Screen - 1242w x 2436h .png
App Icon - 1024 x 1024 .png

App.js
```
<View style={styles.login}>
  <TouchableOpacity onPress={this.login.bind(this)}>
    <Text style={styles.facebookButton}>Sign In with Facebook</Text>
  </TouchableOpacity>
</View>  
```

styles.js
```
login: {
  flex: 1,
  backgroundColor: '#c75c5c',
  alignItems: 'center',
  justifyContent: 'center'
},
facebookButton: {
  backgroundColor: '#3B5998',
  color: '#fff',
  padding: 15,
  paddingLeft: 50,
  paddingRight: 50,
  margin: 25,
},    
```

## Video 13 Submit to App Store
[Apple Certificates](https://developer.apple.com/account/ios/certificate/?teamId=GVXC5FQ2RP)
